<!doctype html>
<html lang="" class="h-full">
<head>
    <meta charset="utf-8"/>
    <meta name="viewport" content="width=device-width, initial-scale=1, viewport-fit=cover"/>
    <meta http-equiv="X-UA-Compatible" content="ie=edge"/>
    <title>{{ page_title }}{% if page_title %} / {% endif %}{{ admin.title }}</title>
    <link rel="preconnect" href="https://unpkg.com">
    <link rel="preconnect" href="https://fonts.gstatic.com" crossorigin>
    <link rel="preconnect" href="https://fonts.googleapis.com" crossorigin>
    <meta name="msapplication-TileColor" content="#206bc4"/>
    <meta name="theme-color" content="rgb(15,23,42)"/>
    <meta name="apple-mobile-web-app-status-bar-style" content="black-translucent"/>
    <meta name="apple-mobile-web-app-capable" content="yes"/>
    <meta name="mobile-web-app-capable" content="yes"/>
    <meta name="HandheldFriendly" content="True"/>
    <meta name="MobileOptimized" content="320"/>
    <meta name="csrf-token" content="{{ csrf_token }}"/>

    <link rel="icon" href="{{ static_url('logo_square.svg') }}" type="image/x-icon"/>
    <link rel="shortcut icon" href="{{ static_url('logo_square.svg') }}" type="image/x-icon"/>

    <link href="{{ static_url('main.css') }}" rel="stylesheet"/>
    <script src="//unpkg.com/htmx.org@1.8.0"
            integrity="sha384-cZuAZ+ZbwkNRnrKi05G/fjBX+azI9DNOkNYysZ0I/X5ZFgsmMiBXgDZof30F5ofc"
            crossorigin="anonymous">

    </script>
    <script defer src="{{ static_url('main.js') }}"></script>

    <style>
        [x-cloak] {
            display: none;
        }
    </style>

    {% block head %}{% endblock %}
    {% block head_scripts %}{% endblock %}
</head>
<body class="antialiased h-full" x-data="{}" x-cloak>
{% block main %}{% endblock %}
{% block page_scripts %}{% endblock %}
<script>
    window.__TOASTS__ = {{ flash_messages|list|tojson }};
</script>

<div id="modals"></div>
</body>
</html>
