{% extends 'ohmyadmin/base_auth.html' %}
{% import 'ohmyadmin/lib/forms.html' as forms %}

{% block content %}
    <form method="post" class="bg-white p-5 rounded-md shadow-sm">
        <p class="text-2xl mb-8 font-medium">{{ _('Sign in to your account', domain='ohmyadmin') }}</p>
        {{ forms.form_field(form.identity) }}
        {{ forms.form_field(form.password) }}
        <div>
            <button class="btn btn-primary" type="submit">
                {{ _('Login', domain='ohmyadmin') }}
            </button>
        </div>
    </form>
{% endblock %}
