{% extends 'ohmyadmin/app.html' %}
{% import 'ohmyadmin/lib/forms.html' as forms %}

{% block content %}
    <form method="post" enctype="multipart/form-data" hx-boost="true">
        <div>
            {{ form_layout.render(request) }}
        </div>
        <div class="form-actions">
            {% block form_actions %}
                {% for form_action in form_actions %}
                    {{ form_action.render(request) }}
                {% endfor %}
            {% endblock %}
        </div>
    </form>
{% endblock %}
