<!-- search form -->
<form method="get"
      role="search"
      action="{{ request.url }}"
      hx-get="{{ request.url.remove_query_params(page.search_param) }}"
>
    <input type="search" incremental
           value="{{ search_term }}"
           name="{{ page.search_param }}"
           hx-get="{{ request.url.remove_query_params(page.search_param) }}"
           hx-trigger="keyup changed delay:300ms, search delay:300ms"
           placeholder="{{ page.get_search_placeholder(request) }}"
    >
</form>
<!-- end search form -->
