<section class="card">
    {% if layout.label or layout.description %}
        <header>
            {% if layout.label %}
                <h3>{{ layout.label }}</h3>
            {% endif %}
            {% if layout.description %}
                <p>{{ layout.description }}</p>
            {% endif %}
        </header>
    {% endif %}
    <main>
        {% for child in layout.children %}
            {{ child.render(request) }}
        {% endfor %}
    </main>
</section>
