{% extends 'ohmyadmin/app.html' %}

{% block content %}
    <form method="post">
        {% trans %}
            You are about to delete "{{ object }}" and related objects.<br>
            Are you sure you want to continue?
        {% endtrans %}
        <div class="form-actions">
            <button type="submit" class="btn btn-danger">{{ _('Yes, delete it', domain='ohmyadmin') }}</button>
            <a href="{{ cancel_url }}" class="btn btn-text">{{ _('Cancel', domain='ohmyadmin') }}</a>
        </div>
    </form>
{% endblock %}
