<a href="{{ item.url }}" class="main-menu-item {{ 'active' if is_active else '' }}">
    <span class="h-5 w-5 inline-block">{{ tabler_icon(item.icon) if item.icon else '' }}</span>
    {{ item.text }}
</a>
