<section class="side-section">
    <aside>
        <h3>{{ layout.label }}</h3>
        {% if layout.description %}
            <p>{{ layout.description }}</p>
        {% endif %}
    </aside>
    <main>
        {% for child in layout.children %}
            {{ child.render(request) }}
        {% endfor %}
    </main>
</section>
