{% extends 'ohmyadmin/metrics/base.html' %}

{% block content %}
    <div class="text-2xl font-medium mb-5" data-test="progress-current">
        {{ percent }}%
    </div>
    <div class="progress progress-lg" data-test="progress-value">
        <div class="progress-bar bg-{{ metric.progress_color }}-500" style="width: {{ percent }}%"></div>
    </div>
{% endblock %}
