<a href="{{ action.resolve(request, object) }}" class="list-menu-item" data-test="object-action">
    <span class="h-5 w-5">{{ tabler_icon(action.icon) if action.icon else '' }}</span>
    <span>{{ action.label }}</span>
</a>
