<button class="btn" @click="showFilterBar = !showFilterBar" data-test="filters-trigger">
    {{ tabler_icon('filter') }}
    <template x-if="selectedFiltersCount">
        <span class="badge badge-blue" x-text="selectedFiltersCount"></span>
    </template>
    <span class="transition duration-140 ease transform" :class="{'rotate-90': showFilterBar}">
        {{ tabler_icon('chevron-right') }}
    </span>
</button>
