<div class="card p-4 col-span-{{ metric.size }} overflow-y-hidden min-h-40 bg-white" data-test="metric">
    <header class="flex items-start justify-between mb-3">
        <div class="font-semibold text-gray-600">{{ metric.label }}</div>
    </header>
    <main class="flex flex-col">
        {% block content %}{% endblock %}
    </main>
</div>
