{% extends 'ohmyadmin/metrics/base.html' %}

{% block content %}
    {% if value.current_value is not none %}
        <div class="text-2xl font-medium mb-2" data-test="trend-current">
            {{ value.current_value }}
        </div>
    {% endif %}
    <div class="-mx-4 h-full -mb-4">
        <script type="application/json" id="{{ metric.slug }}">{{ value.series|tojson }}</script>
        <x-trend-metric-chart series="{{ metric.slug }}"></x-trend-metric-chart>
    </div>
{% endblock %}
