{% extends 'ohmyadmin/filters/base_indicator.html' %}

{% block content %}
    <span class="text-amber-700">{{ _('is in', domain='ohmyadmin') }}</span> {{ indicator.value|join(', ') }}
{% endblock %}
